# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1.32.3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from kubernetes_asyncio.client.configuration import Configuration


class V1alpha1MutatingAdmissionPolicySpec(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'failure_policy': 'str',
        'match_conditions': 'list[V1alpha1MatchCondition]',
        'match_constraints': 'V1alpha1MatchResources',
        'mutations': 'list[V1alpha1Mutation]',
        'param_kind': 'V1alpha1ParamKind',
        'reinvocation_policy': 'str',
        'variables': 'list[V1alpha1Variable]'
    }

    attribute_map = {
        'failure_policy': 'failurePolicy',
        'match_conditions': 'matchConditions',
        'match_constraints': 'matchConstraints',
        'mutations': 'mutations',
        'param_kind': 'paramKind',
        'reinvocation_policy': 'reinvocationPolicy',
        'variables': 'variables'
    }

    def __init__(self, failure_policy=None, match_conditions=None, match_constraints=None, mutations=None, param_kind=None, reinvocation_policy=None, variables=None, local_vars_configuration=None):  # noqa: E501
        """V1alpha1MutatingAdmissionPolicySpec - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default()
        self.local_vars_configuration = local_vars_configuration

        self._failure_policy = None
        self._match_conditions = None
        self._match_constraints = None
        self._mutations = None
        self._param_kind = None
        self._reinvocation_policy = None
        self._variables = None
        self.discriminator = None

        if failure_policy is not None:
            self.failure_policy = failure_policy
        if match_conditions is not None:
            self.match_conditions = match_conditions
        if match_constraints is not None:
            self.match_constraints = match_constraints
        if mutations is not None:
            self.mutations = mutations
        if param_kind is not None:
            self.param_kind = param_kind
        if reinvocation_policy is not None:
            self.reinvocation_policy = reinvocation_policy
        if variables is not None:
            self.variables = variables

    @property
    def failure_policy(self):
        """Gets the failure_policy of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501

        failurePolicy defines how to handle failures for the admission policy. Failures can occur from CEL expression parse errors, type check errors, runtime errors and invalid or mis-configured policy definitions or bindings.  A policy is invalid if paramKind refers to a non-existent Kind. A binding is invalid if paramRef.name refers to a non-existent resource.  failurePolicy does not define how validations that evaluate to false are handled.  Allowed values are Ignore or Fail. Defaults to Fail.  # noqa: E501

        :return: The failure_policy of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501
        :rtype: str
        """
        return self._failure_policy

    @failure_policy.setter
    def failure_policy(self, failure_policy):
        """Sets the failure_policy of this V1alpha1MutatingAdmissionPolicySpec.

        failurePolicy defines how to handle failures for the admission policy. Failures can occur from CEL expression parse errors, type check errors, runtime errors and invalid or mis-configured policy definitions or bindings.  A policy is invalid if paramKind refers to a non-existent Kind. A binding is invalid if paramRef.name refers to a non-existent resource.  failurePolicy does not define how validations that evaluate to false are handled.  Allowed values are Ignore or Fail. Defaults to Fail.  # noqa: E501

        :param failure_policy: The failure_policy of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501
        :type failure_policy: str
        """

        self._failure_policy = failure_policy

    @property
    def match_conditions(self):
        """Gets the match_conditions of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501

        matchConditions is a list of conditions that must be met for a request to be validated. Match conditions filter requests that have already been matched by the matchConstraints. An empty list of matchConditions matches all requests. There are a maximum of 64 match conditions allowed.  If a parameter object is provided, it can be accessed via the `params` handle in the same manner as validation expressions.  The exact matching logic is (in order):   1. If ANY matchCondition evaluates to FALSE, the policy is skipped.   2. If ALL matchConditions evaluate to TRUE, the policy is evaluated.   3. If any matchCondition evaluates to an error (but none are FALSE):      - If failurePolicy=Fail, reject the request      - If failurePolicy=Ignore, the policy is skipped  # noqa: E501

        :return: The match_conditions of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501
        :rtype: list[V1alpha1MatchCondition]
        """
        return self._match_conditions

    @match_conditions.setter
    def match_conditions(self, match_conditions):
        """Sets the match_conditions of this V1alpha1MutatingAdmissionPolicySpec.

        matchConditions is a list of conditions that must be met for a request to be validated. Match conditions filter requests that have already been matched by the matchConstraints. An empty list of matchConditions matches all requests. There are a maximum of 64 match conditions allowed.  If a parameter object is provided, it can be accessed via the `params` handle in the same manner as validation expressions.  The exact matching logic is (in order):   1. If ANY matchCondition evaluates to FALSE, the policy is skipped.   2. If ALL matchConditions evaluate to TRUE, the policy is evaluated.   3. If any matchCondition evaluates to an error (but none are FALSE):      - If failurePolicy=Fail, reject the request      - If failurePolicy=Ignore, the policy is skipped  # noqa: E501

        :param match_conditions: The match_conditions of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501
        :type match_conditions: list[V1alpha1MatchCondition]
        """

        self._match_conditions = match_conditions

    @property
    def match_constraints(self):
        """Gets the match_constraints of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501


        :return: The match_constraints of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501
        :rtype: V1alpha1MatchResources
        """
        return self._match_constraints

    @match_constraints.setter
    def match_constraints(self, match_constraints):
        """Sets the match_constraints of this V1alpha1MutatingAdmissionPolicySpec.


        :param match_constraints: The match_constraints of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501
        :type match_constraints: V1alpha1MatchResources
        """

        self._match_constraints = match_constraints

    @property
    def mutations(self):
        """Gets the mutations of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501

        mutations contain operations to perform on matching objects. mutations may not be empty; a minimum of one mutation is required. mutations are evaluated in order, and are reinvoked according to the reinvocationPolicy. The mutations of a policy are invoked for each binding of this policy and reinvocation of mutations occurs on a per binding basis.  # noqa: E501

        :return: The mutations of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501
        :rtype: list[V1alpha1Mutation]
        """
        return self._mutations

    @mutations.setter
    def mutations(self, mutations):
        """Sets the mutations of this V1alpha1MutatingAdmissionPolicySpec.

        mutations contain operations to perform on matching objects. mutations may not be empty; a minimum of one mutation is required. mutations are evaluated in order, and are reinvoked according to the reinvocationPolicy. The mutations of a policy are invoked for each binding of this policy and reinvocation of mutations occurs on a per binding basis.  # noqa: E501

        :param mutations: The mutations of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501
        :type mutations: list[V1alpha1Mutation]
        """

        self._mutations = mutations

    @property
    def param_kind(self):
        """Gets the param_kind of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501


        :return: The param_kind of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501
        :rtype: V1alpha1ParamKind
        """
        return self._param_kind

    @param_kind.setter
    def param_kind(self, param_kind):
        """Sets the param_kind of this V1alpha1MutatingAdmissionPolicySpec.


        :param param_kind: The param_kind of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501
        :type param_kind: V1alpha1ParamKind
        """

        self._param_kind = param_kind

    @property
    def reinvocation_policy(self):
        """Gets the reinvocation_policy of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501

        reinvocationPolicy indicates whether mutations may be called multiple times per MutatingAdmissionPolicyBinding as part of a single admission evaluation. Allowed values are \"Never\" and \"IfNeeded\".  Never: These mutations will not be called more than once per binding in a single admission evaluation.  IfNeeded: These mutations may be invoked more than once per binding for a single admission request and there is no guarantee of order with respect to other admission plugins, admission webhooks, bindings of this policy and admission policies.  Mutations are only reinvoked when mutations change the object after this mutation is invoked. Required.  # noqa: E501

        :return: The reinvocation_policy of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501
        :rtype: str
        """
        return self._reinvocation_policy

    @reinvocation_policy.setter
    def reinvocation_policy(self, reinvocation_policy):
        """Sets the reinvocation_policy of this V1alpha1MutatingAdmissionPolicySpec.

        reinvocationPolicy indicates whether mutations may be called multiple times per MutatingAdmissionPolicyBinding as part of a single admission evaluation. Allowed values are \"Never\" and \"IfNeeded\".  Never: These mutations will not be called more than once per binding in a single admission evaluation.  IfNeeded: These mutations may be invoked more than once per binding for a single admission request and there is no guarantee of order with respect to other admission plugins, admission webhooks, bindings of this policy and admission policies.  Mutations are only reinvoked when mutations change the object after this mutation is invoked. Required.  # noqa: E501

        :param reinvocation_policy: The reinvocation_policy of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501
        :type reinvocation_policy: str
        """

        self._reinvocation_policy = reinvocation_policy

    @property
    def variables(self):
        """Gets the variables of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501

        variables contain definitions of variables that can be used in composition of other expressions. Each variable is defined as a named CEL expression. The variables defined here will be available under `variables` in other expressions of the policy except matchConditions because matchConditions are evaluated before the rest of the policy.  The expression of a variable can refer to other variables defined earlier in the list but not those after. Thus, variables must be sorted by the order of first appearance and acyclic.  # noqa: E501

        :return: The variables of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501
        :rtype: list[V1alpha1Variable]
        """
        return self._variables

    @variables.setter
    def variables(self, variables):
        """Sets the variables of this V1alpha1MutatingAdmissionPolicySpec.

        variables contain definitions of variables that can be used in composition of other expressions. Each variable is defined as a named CEL expression. The variables defined here will be available under `variables` in other expressions of the policy except matchConditions because matchConditions are evaluated before the rest of the policy.  The expression of a variable can refer to other variables defined earlier in the list but not those after. Thus, variables must be sorted by the order of first appearance and acyclic.  # noqa: E501

        :param variables: The variables of this V1alpha1MutatingAdmissionPolicySpec.  # noqa: E501
        :type variables: list[V1alpha1Variable]
        """

        self._variables = variables

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1alpha1MutatingAdmissionPolicySpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1alpha1MutatingAdmissionPolicySpec):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1.32.3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from kubernetes_asyncio.client.configuration import Configuration


class V1ResourceRequirements(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'claims': 'list[V1ResourceClaim]',
        'limits': 'dict(str, str)',
        'requests': 'dict(str, str)'
    }

    attribute_map = {
        'claims': 'claims',
        'limits': 'limits',
        'requests': 'requests'
    }

    def __init__(self, claims=None, limits=None, requests=None, local_vars_configuration=None):  # noqa: E501
        """V1ResourceRequirements - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default()
        self.local_vars_configuration = local_vars_configuration

        self._claims = None
        self._limits = None
        self._requests = None
        self.discriminator = None

        if claims is not None:
            self.claims = claims
        if limits is not None:
            self.limits = limits
        if requests is not None:
            self.requests = requests

    @property
    def claims(self):
        """Gets the claims of this V1ResourceRequirements.  # noqa: E501

        Claims lists the names of resources, defined in spec.resourceClaims, that are used by this container.  This is an alpha field and requires enabling the DynamicResourceAllocation feature gate.  This field is immutable. It can only be set for containers.  # noqa: E501

        :return: The claims of this V1ResourceRequirements.  # noqa: E501
        :rtype: list[V1ResourceClaim]
        """
        return self._claims

    @claims.setter
    def claims(self, claims):
        """Sets the claims of this V1ResourceRequirements.

        Claims lists the names of resources, defined in spec.resourceClaims, that are used by this container.  This is an alpha field and requires enabling the DynamicResourceAllocation feature gate.  This field is immutable. It can only be set for containers.  # noqa: E501

        :param claims: The claims of this V1ResourceRequirements.  # noqa: E501
        :type claims: list[V1ResourceClaim]
        """

        self._claims = claims

    @property
    def limits(self):
        """Gets the limits of this V1ResourceRequirements.  # noqa: E501

        Limits describes the maximum amount of compute resources allowed. More info: https://kubernetes.io/docs/concepts/configuration/manage-resources-containers/  # noqa: E501

        :return: The limits of this V1ResourceRequirements.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._limits

    @limits.setter
    def limits(self, limits):
        """Sets the limits of this V1ResourceRequirements.

        Limits describes the maximum amount of compute resources allowed. More info: https://kubernetes.io/docs/concepts/configuration/manage-resources-containers/  # noqa: E501

        :param limits: The limits of this V1ResourceRequirements.  # noqa: E501
        :type limits: dict(str, str)
        """

        self._limits = limits

    @property
    def requests(self):
        """Gets the requests of this V1ResourceRequirements.  # noqa: E501

        Requests describes the minimum amount of compute resources required. If Requests is omitted for a container, it defaults to Limits if that is explicitly specified, otherwise to an implementation-defined value. Requests cannot exceed Limits. More info: https://kubernetes.io/docs/concepts/configuration/manage-resources-containers/  # noqa: E501

        :return: The requests of this V1ResourceRequirements.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._requests

    @requests.setter
    def requests(self, requests):
        """Sets the requests of this V1ResourceRequirements.

        Requests describes the minimum amount of compute resources required. If Requests is omitted for a container, it defaults to Limits if that is explicitly specified, otherwise to an implementation-defined value. Requests cannot exceed Limits. More info: https://kubernetes.io/docs/concepts/configuration/manage-resources-containers/  # noqa: E501

        :param requests: The requests of this V1ResourceRequirements.  # noqa: E501
        :type requests: dict(str, str)
        """

        self._requests = requests

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1ResourceRequirements):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1ResourceRequirements):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1.32.3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from kubernetes_asyncio.client.configuration import Configuration


class V1beta1DeviceRequestAllocationResult(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'admin_access': 'bool',
        'device': 'str',
        'driver': 'str',
        'pool': 'str',
        'request': 'str'
    }

    attribute_map = {
        'admin_access': 'adminAccess',
        'device': 'device',
        'driver': 'driver',
        'pool': 'pool',
        'request': 'request'
    }

    def __init__(self, admin_access=None, device=None, driver=None, pool=None, request=None, local_vars_configuration=None):  # noqa: E501
        """V1beta1DeviceRequestAllocationResult - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default()
        self.local_vars_configuration = local_vars_configuration

        self._admin_access = None
        self._device = None
        self._driver = None
        self._pool = None
        self._request = None
        self.discriminator = None

        if admin_access is not None:
            self.admin_access = admin_access
        self.device = device
        self.driver = driver
        self.pool = pool
        self.request = request

    @property
    def admin_access(self):
        """Gets the admin_access of this V1beta1DeviceRequestAllocationResult.  # noqa: E501

        AdminAccess indicates that this device was allocated for administrative access. See the corresponding request field for a definition of mode.  This is an alpha field and requires enabling the DRAAdminAccess feature gate. Admin access is disabled if this field is unset or set to false, otherwise it is enabled.  # noqa: E501

        :return: The admin_access of this V1beta1DeviceRequestAllocationResult.  # noqa: E501
        :rtype: bool
        """
        return self._admin_access

    @admin_access.setter
    def admin_access(self, admin_access):
        """Sets the admin_access of this V1beta1DeviceRequestAllocationResult.

        AdminAccess indicates that this device was allocated for administrative access. See the corresponding request field for a definition of mode.  This is an alpha field and requires enabling the DRAAdminAccess feature gate. Admin access is disabled if this field is unset or set to false, otherwise it is enabled.  # noqa: E501

        :param admin_access: The admin_access of this V1beta1DeviceRequestAllocationResult.  # noqa: E501
        :type admin_access: bool
        """

        self._admin_access = admin_access

    @property
    def device(self):
        """Gets the device of this V1beta1DeviceRequestAllocationResult.  # noqa: E501

        Device references one device instance via its name in the driver's resource pool. It must be a DNS label.  # noqa: E501

        :return: The device of this V1beta1DeviceRequestAllocationResult.  # noqa: E501
        :rtype: str
        """
        return self._device

    @device.setter
    def device(self, device):
        """Sets the device of this V1beta1DeviceRequestAllocationResult.

        Device references one device instance via its name in the driver's resource pool. It must be a DNS label.  # noqa: E501

        :param device: The device of this V1beta1DeviceRequestAllocationResult.  # noqa: E501
        :type device: str
        """
        if self.local_vars_configuration.client_side_validation and device is None:  # noqa: E501
            raise ValueError("Invalid value for `device`, must not be `None`")  # noqa: E501

        self._device = device

    @property
    def driver(self):
        """Gets the driver of this V1beta1DeviceRequestAllocationResult.  # noqa: E501

        Driver specifies the name of the DRA driver whose kubelet plugin should be invoked to process the allocation once the claim is needed on a node.  Must be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver.  # noqa: E501

        :return: The driver of this V1beta1DeviceRequestAllocationResult.  # noqa: E501
        :rtype: str
        """
        return self._driver

    @driver.setter
    def driver(self, driver):
        """Sets the driver of this V1beta1DeviceRequestAllocationResult.

        Driver specifies the name of the DRA driver whose kubelet plugin should be invoked to process the allocation once the claim is needed on a node.  Must be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver.  # noqa: E501

        :param driver: The driver of this V1beta1DeviceRequestAllocationResult.  # noqa: E501
        :type driver: str
        """
        if self.local_vars_configuration.client_side_validation and driver is None:  # noqa: E501
            raise ValueError("Invalid value for `driver`, must not be `None`")  # noqa: E501

        self._driver = driver

    @property
    def pool(self):
        """Gets the pool of this V1beta1DeviceRequestAllocationResult.  # noqa: E501

        This name together with the driver name and the device name field identify which device was allocated (`<driver name>/<pool name>/<device name>`).  Must not be longer than 253 characters and may contain one or more DNS sub-domains separated by slashes.  # noqa: E501

        :return: The pool of this V1beta1DeviceRequestAllocationResult.  # noqa: E501
        :rtype: str
        """
        return self._pool

    @pool.setter
    def pool(self, pool):
        """Sets the pool of this V1beta1DeviceRequestAllocationResult.

        This name together with the driver name and the device name field identify which device was allocated (`<driver name>/<pool name>/<device name>`).  Must not be longer than 253 characters and may contain one or more DNS sub-domains separated by slashes.  # noqa: E501

        :param pool: The pool of this V1beta1DeviceRequestAllocationResult.  # noqa: E501
        :type pool: str
        """
        if self.local_vars_configuration.client_side_validation and pool is None:  # noqa: E501
            raise ValueError("Invalid value for `pool`, must not be `None`")  # noqa: E501

        self._pool = pool

    @property
    def request(self):
        """Gets the request of this V1beta1DeviceRequestAllocationResult.  # noqa: E501

        Request is the name of the request in the claim which caused this device to be allocated. Multiple devices may have been allocated per request.  # noqa: E501

        :return: The request of this V1beta1DeviceRequestAllocationResult.  # noqa: E501
        :rtype: str
        """
        return self._request

    @request.setter
    def request(self, request):
        """Sets the request of this V1beta1DeviceRequestAllocationResult.

        Request is the name of the request in the claim which caused this device to be allocated. Multiple devices may have been allocated per request.  # noqa: E501

        :param request: The request of this V1beta1DeviceRequestAllocationResult.  # noqa: E501
        :type request: str
        """
        if self.local_vars_configuration.client_side_validation and request is None:  # noqa: E501
            raise ValueError("Invalid value for `request`, must not be `None`")  # noqa: E501

        self._request = request

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1beta1DeviceRequestAllocationResult):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1beta1DeviceRequestAllocationResult):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1.32.3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from kubernetes_asyncio.client.configuration import Configuration


class V1SubjectRulesReviewStatus(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'evaluation_error': 'str',
        'incomplete': 'bool',
        'non_resource_rules': 'list[V1NonResourceRule]',
        'resource_rules': 'list[V1ResourceRule]'
    }

    attribute_map = {
        'evaluation_error': 'evaluationError',
        'incomplete': 'incomplete',
        'non_resource_rules': 'nonResourceRules',
        'resource_rules': 'resourceRules'
    }

    def __init__(self, evaluation_error=None, incomplete=None, non_resource_rules=None, resource_rules=None, local_vars_configuration=None):  # noqa: E501
        """V1SubjectRulesReviewStatus - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default()
        self.local_vars_configuration = local_vars_configuration

        self._evaluation_error = None
        self._incomplete = None
        self._non_resource_rules = None
        self._resource_rules = None
        self.discriminator = None

        if evaluation_error is not None:
            self.evaluation_error = evaluation_error
        self.incomplete = incomplete
        self.non_resource_rules = non_resource_rules
        self.resource_rules = resource_rules

    @property
    def evaluation_error(self):
        """Gets the evaluation_error of this V1SubjectRulesReviewStatus.  # noqa: E501

        EvaluationError can appear in combination with Rules. It indicates an error occurred during rule evaluation, such as an authorizer that doesn't support rule evaluation, and that ResourceRules and/or NonResourceRules may be incomplete.  # noqa: E501

        :return: The evaluation_error of this V1SubjectRulesReviewStatus.  # noqa: E501
        :rtype: str
        """
        return self._evaluation_error

    @evaluation_error.setter
    def evaluation_error(self, evaluation_error):
        """Sets the evaluation_error of this V1SubjectRulesReviewStatus.

        EvaluationError can appear in combination with Rules. It indicates an error occurred during rule evaluation, such as an authorizer that doesn't support rule evaluation, and that ResourceRules and/or NonResourceRules may be incomplete.  # noqa: E501

        :param evaluation_error: The evaluation_error of this V1SubjectRulesReviewStatus.  # noqa: E501
        :type evaluation_error: str
        """

        self._evaluation_error = evaluation_error

    @property
    def incomplete(self):
        """Gets the incomplete of this V1SubjectRulesReviewStatus.  # noqa: E501

        Incomplete is true when the rules returned by this call are incomplete. This is most commonly encountered when an authorizer, such as an external authorizer, doesn't support rules evaluation.  # noqa: E501

        :return: The incomplete of this V1SubjectRulesReviewStatus.  # noqa: E501
        :rtype: bool
        """
        return self._incomplete

    @incomplete.setter
    def incomplete(self, incomplete):
        """Sets the incomplete of this V1SubjectRulesReviewStatus.

        Incomplete is true when the rules returned by this call are incomplete. This is most commonly encountered when an authorizer, such as an external authorizer, doesn't support rules evaluation.  # noqa: E501

        :param incomplete: The incomplete of this V1SubjectRulesReviewStatus.  # noqa: E501
        :type incomplete: bool
        """
        if self.local_vars_configuration.client_side_validation and incomplete is None:  # noqa: E501
            raise ValueError("Invalid value for `incomplete`, must not be `None`")  # noqa: E501

        self._incomplete = incomplete

    @property
    def non_resource_rules(self):
        """Gets the non_resource_rules of this V1SubjectRulesReviewStatus.  # noqa: E501

        NonResourceRules is the list of actions the subject is allowed to perform on non-resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.  # noqa: E501

        :return: The non_resource_rules of this V1SubjectRulesReviewStatus.  # noqa: E501
        :rtype: list[V1NonResourceRule]
        """
        return self._non_resource_rules

    @non_resource_rules.setter
    def non_resource_rules(self, non_resource_rules):
        """Sets the non_resource_rules of this V1SubjectRulesReviewStatus.

        NonResourceRules is the list of actions the subject is allowed to perform on non-resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.  # noqa: E501

        :param non_resource_rules: The non_resource_rules of this V1SubjectRulesReviewStatus.  # noqa: E501
        :type non_resource_rules: list[V1NonResourceRule]
        """
        if self.local_vars_configuration.client_side_validation and non_resource_rules is None:  # noqa: E501
            raise ValueError("Invalid value for `non_resource_rules`, must not be `None`")  # noqa: E501

        self._non_resource_rules = non_resource_rules

    @property
    def resource_rules(self):
        """Gets the resource_rules of this V1SubjectRulesReviewStatus.  # noqa: E501

        ResourceRules is the list of actions the subject is allowed to perform on resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.  # noqa: E501

        :return: The resource_rules of this V1SubjectRulesReviewStatus.  # noqa: E501
        :rtype: list[V1ResourceRule]
        """
        return self._resource_rules

    @resource_rules.setter
    def resource_rules(self, resource_rules):
        """Sets the resource_rules of this V1SubjectRulesReviewStatus.

        ResourceRules is the list of actions the subject is allowed to perform on resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.  # noqa: E501

        :param resource_rules: The resource_rules of this V1SubjectRulesReviewStatus.  # noqa: E501
        :type resource_rules: list[V1ResourceRule]
        """
        if self.local_vars_configuration.client_side_validation and resource_rules is None:  # noqa: E501
            raise ValueError("Invalid value for `resource_rules`, must not be `None`")  # noqa: E501

        self._resource_rules = resource_rules

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1SubjectRulesReviewStatus):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1SubjectRulesReviewStatus):
            return True

        return self.to_dict() != other.to_dict()



# Copyright 2017 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import math
import re


class TimezoneInfo(datetime.tzinfo):
    def __init__(self, h, m):
        self._name = "UTC"
        if h != 0 and m != 0:
            self._name += "%+03d:%2d" % (h, m)
        self._delta = datetime.timedelta(hours=h, minutes=math.copysign(m, h))

    def utcoffset(self, dt):
        return self._delta

    def tzname(self, dt):
        return self._name

    def dst(self, dt):
        return datetime.timedelta(0)


UTC = TimezoneInfo(0, 0)

# ref https://www.ietf.org/rfc/rfc3339.txt
_re_rfc3339 = re.compile(r"(\d\d\d\d)-(\d\d)-(\d\d)"        # full-date
                         r"[ Tt]"                           # Separator
                         r"(\d\d):(\d\d):(\d\d)([.,]\d+)?"  # partial-time
                         r"([zZ ]|[-+]\d\d?:\d\d)?",        # time-offset
                         re.VERBOSE + re.IGNORECASE)
_re_timezone = re.compile(r"([-+])(\d\d?):?(\d\d)?")


MICROSEC_PER_SEC = 1000000


def parse_rfc3339(s):
    if isinstance(s, datetime.datetime):
        # no need to parse it, just make sure it has a timezone.
        if not s.tzinfo:
            return s.replace(tzinfo=UTC)
        return s
    groups = _re_rfc3339.search(s).groups()
    dt = [0] * 7
    for x in range(6):
        dt[x] = int(groups[x])
    us = 0
    if groups[6] is not None:
        partial_sec = float(groups[6].replace(",", "."))
        us = int(MICROSEC_PER_SEC * partial_sec)
    tz = UTC
    if groups[7] is not None and groups[7] != 'Z' and groups[7] != 'z':
        tz_groups = _re_timezone.search(groups[7]).groups()
        hour = int(tz_groups[1])
        minute = 0
        if tz_groups[0] == "-":
            hour *= -1
        if tz_groups[2]:
            minute = int(tz_groups[2])
        tz = TimezoneInfo(hour, minute)
    return datetime.datetime(
        year=dt[0], month=dt[1], day=dt[2],
        hour=dt[3], minute=dt[4], second=dt[5],
        microsecond=us, tzinfo=tz)


def format_rfc3339(date_time):
    if date_time.tzinfo is None:
        date_time = date_time.replace(tzinfo=UTC)
    date_time = date_time.astimezone(UTC)
    return date_time.strftime('%Y-%m-%dT%H:%M:%SZ')



# Copyright 2018 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import sys
from unittest import IsolatedAsyncioTestCase
from unittest.mock import ANY, AsyncMock, patch

from .config_exception import ConfigException
from .exec_provider import ExecProvider
from .kube_config import ConfigNode


class ExecProviderTest(IsolatedAsyncioTestCase):

    def setUp(self):
        self.input_ok = ConfigNode('test', {
            'command': 'aws-iam-authenticator',
            'args': ['token', '-i', 'dummy'],
            'apiVersion': 'client.authentication.k8s.io/v1beta1',
            'env': None
        })
        self.output_ok = """
        {
            "apiVersion": "client.authentication.k8s.io/v1beta1",
            "kind": "ExecCredential",
            "status": {
                "token": "dummy"
            }
        }
        """

        process_patch = patch('kubernetes_asyncio.config.exec_provider.asyncio.create_subprocess_exec')
        self.exec_mock = process_patch.start()
        self.process_mock = self.exec_mock.return_value
        self.process_mock.stdout.read = AsyncMock(return_value=self.output_ok)
        self.process_mock.stderr.read = AsyncMock(return_value='')
        self.process_mock.wait = AsyncMock(return_value=0)

    def tearDown(self):
        patch.stopall()

    def test_missing_input_keys(self):
        exec_configs = [{}, {'command': ''}, {'apiVersion': ''}]
        for exec_config in exec_configs:
            with self.assertRaises(ConfigException) as context:
                ExecProvider(exec_config)
            self.assertIn('exec: malformed request. missing key',
                          context.exception.args[0])

    async def test_error_code_returned(self):
        self.process_mock.stdout.read.return_value = ''
        self.process_mock.wait.return_value = 1
        with self.assertRaisesRegex(ConfigException, 'exec: process returned 1'):
            ep = ExecProvider(self.input_ok)
            await ep.run()

    async def test_nonjson_output_returned(self):
        self.process_mock.stdout.read.return_value = ''
        with self.assertRaisesRegex(ConfigException, 'exec: failed to decode process output'):
            ep = ExecProvider(self.input_ok)
            await ep.run()

    async def test_missing_output_keys(self):
        outputs = [
            """
            {
                "kind": "ExecCredential",
                "status": {
                    "token": "dummy"
                }
            }
            """, """
            {
                "apiVersion": "client.authentication.k8s.io/v1beta1",
                "status": {
                    "token": "dummy"
                }
            }
            """, """
            {
                "apiVersion": "client.authentication.k8s.io/v1beta1",
                "kind": "ExecCredential"
            }
            """
        ]
        for output in outputs:
            self.process_mock.stdout.read.return_value = output
            with self.assertRaisesRegex(ConfigException, 'exec: malformed response. missing key'):
                ep = ExecProvider(self.input_ok)
                await ep.run()

    async def test_mismatched_api_version(self):
        wrong_api_version = 'client.authentication.k8s.io/v1'
        output = """
        {
            "apiVersion": "%s",
            "kind": "ExecCredential",
            "status": {
                "token": "dummy"
            }
        }
        """ % wrong_api_version
        self.process_mock.stdout.read.return_value = output
        with self.assertRaisesRegex(
                ConfigException,
                'exec: plugin api version {} does not match'.format(wrong_api_version)):
            ep = ExecProvider(self.input_ok)
            await ep.run()

    async def test_ok_01(self):
        ep = ExecProvider(self.input_ok)
        result = await ep.run()
        self.assertTrue(isinstance(result, dict))
        self.assertTrue('token' in result)
        self.exec_mock.assert_called_once_with('aws-iam-authenticator', 'token', '-i', 'dummy',
                                               env=ANY, stderr=-1, stdin=None, stdout=-1)
        self.process_mock.stdout.read.assert_awaited_once()
        self.process_mock.stderr.read.assert_awaited_once()
        self.process_mock.wait.assert_awaited_once()

    async def test_ok_with_args(self):
        input_ok = ConfigNode('test', {
            'command': 'aws-iam-authenticator',
            'apiVersion': 'client.authentication.k8s.io/v1beta1',
            'args': ['token', '-i', 'dummy', '--mock', '90']
        })
        ep = ExecProvider(input_ok)
        result = await ep.run()
        self.assertTrue(isinstance(result, dict))
        self.assertTrue('token' in result)
        self.exec_mock.assert_called_once_with('aws-iam-authenticator', 'token', '-i', 'dummy', '--mock', '90',
                                               env=ANY, stderr=-1, stdin=None, stdout=-1)
        self.process_mock.stdout.read.assert_awaited_once()
        self.process_mock.stderr.read.assert_awaited_once()
        self.process_mock.wait.assert_awaited_once()

    async def test_ok_with_env(self):

        input_ok = ConfigNode('test', {
            'command': 'aws-iam-authenticator',
            'apiVersion': 'client.authentication.k8s.io/v1beta1',
            'args': ['token', '-i', 'dummy'],
            'env': [{'name': 'EXEC_PROVIDER_ENV_NAME',
                     'value': 'EXEC_PROVIDER_ENV_VALUE'}]})

        ep = ExecProvider(input_ok)
        result = await ep.run()
        self.assertTrue(isinstance(result, dict))
        self.assertTrue('token' in result)

        env_used = self.exec_mock.await_args_list[0][1]['env']
        self.assertEqual(env_used['EXEC_PROVIDER_ENV_NAME'], 'EXEC_PROVIDER_ENV_VALUE')
        self.assertEqual(json.loads(env_used['KUBERNETES_EXEC_INFO']), {'apiVersion':
                                                                        'client.authentication.k8s.io/v1beta1',
                                                                        'kind': 'ExecCredential',
                                                                        'spec': {'interactive': sys.stdout.isatty()}})
        self.exec_mock.assert_called_once_with('aws-iam-authenticator', 'token', '-i', 'dummy',
                                               env=ANY, stderr=-1, stdin=None, stdout=-1)
        self.process_mock.stdout.read.assert_awaited_once()
        self.process_mock.stderr.read.assert_awaited_once()
        self.process_mock.wait.assert_awaited_once()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from six.moves.urllib.parse import urlencode, urlparse, urlunparse

from kubernetes_asyncio.client import ApiClient
from kubernetes_asyncio.client.rest import RESTResponse

STDIN_CHANNEL = 0
STDOUT_CHANNEL = 1
STDERR_CHANNEL = 2
ERROR_CHANNEL = 3
RESIZE_CHANNEL = 4


def get_websocket_url(url):
    parsed_url = urlparse(url)
    parts = list(parsed_url)
    if parsed_url.scheme == 'http':
        parts[0] = 'ws'
    elif parsed_url.scheme == 'https':
        parts[0] = 'wss'
    return urlunparse(parts)


class WsResponse(RESTResponse):

    def __init__(self, status, data):
        self.status = status
        self.data = data
        self.headers = {}
        self.reason = None

    def getheaders(self):
        return self.headers

    def getheader(self, name, default=None):
        return self.headers.get(name, default)


class WsApiClient(ApiClient):

    def __init__(self, configuration=None, header_name=None, header_value=None,
                 cookie=None, pool_threads=1, heartbeat=None):
        super().__init__(configuration, header_name, header_value, cookie, pool_threads)
        self.heartbeat = heartbeat

    @classmethod
    def parse_error_data(cls, error_data):
        """
        Parse data received on ERROR_CHANNEL and return the command exit code.
        """
        error_data_json = json.loads(error_data)
        if error_data_json.get("status") == "Success":
            return 0
        return int(error_data_json["details"]["causes"][0]['message'])

    async def request(self, method, url, query_params=None, headers=None,
                      post_params=None, body=None, _preload_content=True,
                      _request_timeout=None):

        # Expand command parameter list to indivitual command params
        if query_params:
            new_query_params = []
            for key, value in query_params:
                if key == 'command' and isinstance(value, list):
                    for command in value:
                        new_query_params.append((key, command))
                else:
                    new_query_params.append((key, value))
            query_params = new_query_params

        if headers is None:
            headers = {}
        if 'sec-websocket-protocol' not in headers:
            headers['sec-websocket-protocol'] = 'v4.channel.k8s.io'

        if query_params:
            url += '?' + urlencode(query_params)

        url = get_websocket_url(url)

        if _preload_content:

            resp_all = ''
            async with self.rest_client.pool_manager.ws_connect(url, headers=headers, heartbeat=self.heartbeat) as ws:
                async for msg in ws:
                    msg = msg.data.decode('utf-8')
                    if len(msg) > 1:
                        channel = ord(msg[0])
                        data = msg[1:]
                        if data:
                            if channel in [STDOUT_CHANNEL, STDERR_CHANNEL]:
                                resp_all += data

            return WsResponse(200, resp_all.encode('utf-8'))

        else:

            return self.rest_client.pool_manager.ws_connect(url, headers=headers, heartbeat=self.heartbeat)



# Copyright 2019 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio
import unittest
import uuid

from kubernetes_asyncio.client import api_client
from kubernetes_asyncio.dynamic import DynamicClient
from kubernetes_asyncio.dynamic.exceptions import ResourceNotFoundError
from kubernetes_asyncio.dynamic.resource import ResourceField, ResourceInstance
from kubernetes_asyncio.e2e_test import base


def short_uuid():
    id_ = str(uuid.uuid4())
    return id_[-12:]


class TestDynamicClient(unittest.IsolatedAsyncioTestCase):

    @classmethod
    def setUpClass(cls):
        cls.config = base.get_e2e_configuration()

    async def test_cluster_custom_resources(self):
        async with api_client.ApiClient(configuration=self.config) as apic:
            client = await DynamicClient(apic)

            with self.assertRaises(ResourceNotFoundError):
                await client.resources.get(api_version='apps.example.com/v1', kind='ClusterChangeMe')

            crd_api = await client.resources.get(
                api_version='apiextensions.k8s.io/v1',
                kind='CustomResourceDefinition')
            name = 'clusterchangemes.apps.example.com'
            crd_manifest = {
                "apiVersion": "apiextensions.k8s.io/v1",
                "kind": "CustomResourceDefinition",
                "metadata": {
                    "name": name,
                },
                "spec": {
                    "group": "apps.example.com",
                    "names": {
                        "kind": "ClusterChangeMe",
                        "listKind": "ClusterChangeMeList",
                        "plural": "clusterchangemes",
                        "singular": "clusterchangeme",
                    },
                    "scope": "Cluster",
                    "versions": [
                        {
                            "name": "v1",
                            "served": True,
                            "storage": True,
                            "schema": {
                                "openAPIV3Schema": {
                                    "type": "object",
                                    "properties": {
                                        "spec": {
                                            "type": "object",
                                            "properties": {"size": {"type": "integer"}},
                                        }
                                    },
                                }
                            },
                        }
                    ],
                },
            }

            resp = await crd_api.create(crd_manifest)
            self.assertEqual(name, resp.metadata.name)
            self.assertTrue(resp.status)

            resp = await crd_api.get(
                name=name,
            )
            self.assertEqual(name, resp.metadata.name)
            self.assertTrue(resp.status)

            try:
                await client.resources.get(api_version='apps.example.com/v1', kind='ClusterChangeMe')
            except ResourceNotFoundError:
                # Need to wait a sec for the discovery layer to get updated
                await asyncio.sleep(2)
            changeme_api = await client.resources.get(
                api_version='apps.example.com/v1', kind='ClusterChangeMe')
            resp = await changeme_api.get()
            self.assertEqual(resp.items, [])
            changeme_name = 'custom-resource' + short_uuid()
            changeme_manifest = {
                'apiVersion': 'apps.example.com/v1',
                'kind': 'ClusterChangeMe',
                'metadata': {
                    'name': changeme_name,
                },
                'spec': {}
            }

            resp = await changeme_api.create(body=changeme_manifest)
            self.assertEqual(resp.metadata.name, changeme_name)

            # watch with timeout
            count = 0
            async for _ in client.watch(changeme_api, timeout=3, namespace="default", name=changeme_name):
                count += 1
            self.assertTrue(count > 0, msg="no events received for watch")

            # without timeout, should be longer than the previous check
            async def _watch_no_timeout():
                async for _ in client.watch(changeme_api, namespace="default", name=changeme_name):
                    pass
            with self.assertRaises(asyncio.exceptions.TimeoutError):
                await asyncio.wait_for(_watch_no_timeout(), timeout=5)

            resp = await changeme_api.get(name=changeme_name)
            self.assertEqual(resp.metadata.name, changeme_name)

            changeme_manifest['spec']['size'] = 3
            resp = await changeme_api.patch(
                body=changeme_manifest,
                content_type='application/merge-patch+json'
            )
            self.assertEqual(resp.spec.size, 3)

            resp = await changeme_api.get(name=changeme_name)
            self.assertEqual(resp.spec.size, 3)

            resp = await changeme_api.get()
            self.assertEqual(len(resp.items), 1)

            await changeme_api.delete(name=changeme_name)

            resp = await changeme_api.get()
            self.assertEqual(len(resp.items), 0)

            await crd_api.delete(name=name)

            await asyncio.sleep(2)
            await client.resources.invalidate_cache()
            with self.assertRaises(ResourceNotFoundError):
                await client.resources.get(api_version='apps.example.com/v1', kind='ClusterChangeMe')

    async def test_namespaced_custom_resources(self):
        async with api_client.ApiClient(configuration=self.config) as apic:
            client = await DynamicClient(apic)

            with self.assertRaises(ResourceNotFoundError):
                await client.resources.get(api_version='apps.example.com/v1', kind='ChangeMe')

            crd_api = await client.resources.get(
                api_version='apiextensions.k8s.io/v1',
                kind='CustomResourceDefinition')
            name = 'clusterchangemes.apps.example.com'
            crd_manifest = {
                "apiVersion": "apiextensions.k8s.io/v1",
                "kind": "CustomResourceDefinition",
                "metadata": {
                    "name": name,
                },
                "spec": {
                    "group": "apps.example.com",
                    "names": {
                        "kind": "ClusterChangeMe",
                        "listKind": "ClusterChangeMeList",
                        "plural": "clusterchangemes",
                        "singular": "clusterchangeme",
                    },
                    "scope": "Namespaced",
                    "versions": [
                        {
                            "name": "v1",
                            "served": True,
                            "storage": True,
                            "schema": {
                                "openAPIV3Schema": {
                                    "type": "object",
                                    "properties": {
                                        "spec": {
                                            "type": "object",
                                            "properties": {"size": {"type": "integer"}},
                                        }
                                    },
                                }
                            },
                        }
                    ],
                },
            }

            resp = await crd_api.create(crd_manifest)
            self.assertEqual(name, resp.metadata.name)
            self.assertTrue(resp.status)

            resp = await crd_api.get(
                name=name,
            )
            self.assertEqual(name, resp.metadata.name)
            self.assertTrue(resp.status)

            try:
                await client.resources.get(api_version='apps.example.com/v1', kind='ClusterChangeMe')
            except ResourceNotFoundError:
                # Need to wait a sec for the discovery layer to get updated
                await asyncio.sleep(2)
            changeme_api = await client.resources.get(
                api_version='apps.example.com/v1', kind='ClusterChangeMe')
            resp = await changeme_api.get()
            self.assertEqual(resp.items, [])
            changeme_name = 'custom-resource' + short_uuid()
            changeme_manifest = {
                'apiVersion': 'apps.example.com/v1',
                'kind': 'ClusterChangeMe',
                'metadata': {
                    'name': changeme_name,
                },
                'spec': {}
            }

            resp = await changeme_api.create(body=changeme_manifest, namespace='default')
            self.assertEqual(resp.metadata.name, changeme_name)

            resp = await changeme_api.get(name=changeme_name, namespace='default')
            self.assertEqual(resp.metadata.name, changeme_name)

            changeme_manifest['spec']['size'] = 3
            resp = await changeme_api.patch(
                body=changeme_manifest,
                namespace='default',
                content_type='application/merge-patch+json'
            )
            self.assertEqual(resp.spec.size, 3)

            resp = await changeme_api.get(name=changeme_name, namespace='default')
            self.assertEqual(resp.spec.size, 3)

            resp = await changeme_api.get(namespace='default')
            self.assertEqual(len(resp.items), 1)

            resp = await changeme_api.get()
            self.assertEqual(len(resp.items), 1)

            await changeme_api.delete(name=changeme_name, namespace='default')

            resp = await changeme_api.get(namespace='default')
            self.assertEqual(len(resp.items), 0)

            resp = await changeme_api.get()
            self.assertEqual(len(resp.items), 0)

            await crd_api.delete(name=name)

            await asyncio.sleep(2)
            await client.resources.invalidate_cache()
            with self.assertRaises(ResourceNotFoundError):
                await client.resources.get(api_version='apps.example.com/v1', kind='ChangeMe')

    async def test_service_apis(self):
        async with api_client.ApiClient(configuration=self.config) as apic:
            client = await DynamicClient(apic)
            api = await client.resources.get(api_version='v1', kind='Service')

            name = 'frontend-' + short_uuid()
            service_manifest = {'apiVersion': 'v1',
                                'kind': 'Service',
                                'metadata': {'labels': {'name': name},
                                             'name': name,
                                             'resourceversion': 'v1'},
                                'spec': {'ports': [{'name': 'port',
                                                    'port': 80,
                                                    'protocol': 'TCP',
                                                    'targetPort': 80}],
                                         'selector': {'name': name}}}

            resp = await api.create(
                body=service_manifest,
                namespace='default'
            )
            self.assertEqual(name, resp.metadata.name)
            self.assertTrue(resp.status)

            resp = await api.get(
                name=name,
                namespace='default'
            )
            self.assertEqual(name, resp.metadata.name)
            self.assertTrue(resp.status)

            service_manifest['spec']['ports'] = [{'name': 'new',
                                                  'port': 8080,
                                                  'protocol': 'TCP',
                                                  'targetPort': 8080}]
            resp = await api.patch(
                body=service_manifest,
                name=name,
                namespace='default'
            )
            self.assertEqual(2, len(resp.spec.ports))
            self.assertTrue(resp.status)

            await api.delete(name=name, body={}, namespace='default')

    async def test_replication_controller_apis(self):
        async with api_client.ApiClient(configuration=self.config) as apic:
            client = await DynamicClient(apic)

            api = await client.resources.get(
                api_version='v1', kind='ReplicationController')

            name = 'frontend-' + short_uuid()
            rc_manifest = {
                'apiVersion': 'v1',
                'kind': 'ReplicationController',
                'metadata': {'labels': {'name': name},
                             'name': name},
                'spec': {'replicas': 2,
                         'selector': {'name': name},
                         'template': {'metadata': {
                             'labels': {'name': name}},
                             'spec': {'containers': [{
                                 'image': 'nginx',
                                 'name': 'nginx',
                                 'ports': [{'containerPort': 80,
                                            'protocol': 'TCP'}]}]}}}}

            resp = await api.create(
                body=rc_manifest, namespace='default')
            self.assertEqual(name, resp.metadata.name)
            self.assertEqual(2, resp.spec.replicas)

            resp = await api.get(
                name=name, namespace='default')
            self.assertEqual(name, resp.metadata.name)
            self.assertEqual(2, resp.spec.replicas)

            await api.delete(
                name=name,
                namespace='default',
                propagation_policy='Background')

    async def test_configmap_apis(self):
        async with api_client.ApiClient(configuration=self.config) as apic:
            client = await DynamicClient(apic)
            api = await client.resources.get(api_version='v1', kind='ConfigMap')

            name = 'test-configmap-' + short_uuid()
            test_configmap = {
                "kind": "ConfigMap",
                "apiVersion": "v1",
                "metadata": {
                    "name": name,
                    "labels": {
                        "e2e-test": "true",
                    },
                },
                "data": {
                    "config.json": "{\"command\":\"/usr/bin/mysqld_safe\"}",
                    "frontend.cnf": "[mysqld]\nbind-address = 10.0.0.3\n"
                }
            }

            resp = await api.create(
                body=test_configmap, namespace='default'
            )
            self.assertEqual(name, resp.metadata.name)

            resp = await api.get(
                name=name, namespace='default', label_selector="e2e-test=true")
            self.assertEqual(name, resp.metadata.name)

            count = 0
            async for _ in client.watch(api, timeout=3, namespace="default", name=name):
                count += 1
            self.assertTrue(count > 0, msg="no events received for watch")

            test_configmap['data']['config.json'] = "{}"
            await api.patch(name=name, namespace='default', body=test_configmap)

            await api.delete(name=name, body={}, namespace='default')

            resp = await api.get(
                namespace='default',
                pretty=True,
                label_selector="e2e-test=true")
            self.assertEqual([], resp.items)

    async def test_node_apis(self):
        async with api_client.ApiClient(configuration=self.config) as apic:
            client = await DynamicClient(apic)

            api = await client.resources.get(api_version='v1', kind='Node')
            nodes = await api.get()
            for item in nodes.items:
                node = await api.get(name=item.metadata.name)
                self.assertTrue(len(dict(node.metadata.labels)) > 0)

    # test_node_apis_partial_object_metadata lists all nodes in the cluster,
    # but only retrieves object metadata
    async def test_node_apis_partial_object_metadata(self):
        async with api_client.ApiClient(configuration=self.config) as apic:
            client = await DynamicClient(apic)
            api = await client.resources.get(api_version='v1', kind='Node')

            params = {
                'header_params': {
                    'Accept': 'application/json;as=PartialObjectMetadataList;v=v1;g=meta.k8s.io'}}
            resp = await api.get(**params)
            self.assertEqual('PartialObjectMetadataList', resp.kind)
            self.assertEqual('meta.k8s.io/v1', resp.apiVersion)

            params = {
                'header_params': {
                    'aCcePt': 'application/json;as=PartialObjectMetadataList;v=v1;g=meta.k8s.io'}}
            resp = await api.get(**params)
            self.assertEqual('PartialObjectMetadataList', resp.kind)
            self.assertEqual('meta.k8s.io/v1', resp.apiVersion)

    async def test_server_side_apply_api(self):
        async with api_client.ApiClient(configuration=self.config) as apic:
            client = await DynamicClient(apic)
            api = await client.resources.get(api_version='v1', kind='Pod')

            name = 'pod-' + short_uuid()
            pod_manifest = {
                'apiVersion': 'apps/v1',
                'kind': 'Deployment',
                'metadata': {'labels': {'name': name},
                             'name': name},
                'spec': {'template': {'spec': {'containers': [{
                    'image': 'nginx',
                    'name': 'nginx',
                    'ports': [{'containerPort': 80,
                               'protocol': 'TCP'}]}]}}}}

            resp = await api.server_side_apply(
                namespace='default', body=pod_manifest,
                field_manager='kubernetes-unittests', dry_run="All")
        self.assertEqual('kubernetes-unittests', resp.metadata.managedFields[0].manager)


class TestDynamicClientSerialization(unittest.IsolatedAsyncioTestCase):

    @classmethod
    def setUpClass(cls):
        cls.config = base.get_e2e_configuration()
        cls.pod_manifest = {
            'apiVersion': 'v1',
            'kind': 'Pod',
            'metadata': {'name': 'foo-pod'},
            'spec': {'containers': [{'name': "main", 'image': "busybox"}]},
        }

    async def test_dict_type(self):
        async with api_client.ApiClient(configuration=self.config) as apic:
            client = await DynamicClient(apic)
            self.assertEqual(client.serialize_body(self.pod_manifest), self.pod_manifest)

    async def test_resource_instance_type(self):
        async with api_client.ApiClient(configuration=self.config) as apic:
            client = await DynamicClient(apic)
            inst = ResourceInstance(client, self.pod_manifest)
            self.assertEqual(client.serialize_body(inst), self.pod_manifest)

    async def test_resource_field(self):
        """`ResourceField` is a special type which overwrites `__getattr__` method to return `None`
        when a non-existent attribute was accessed. which means it can pass any `hasattr(...)` tests.
        """
        params = {
            "foo": "bar",
            "self": True
        }
        res = ResourceField(params)
        self.assertEqual(res["foo"], params["foo"])
        self.assertEqual(res["self"], params["self"])

        # method will return original object when it doesn't know how to proceed
        async with api_client.ApiClient(configuration=self.config) as apic:
            client = await DynamicClient(apic)
            serialized = client.serialize_body(res)
        self.assertDictEqual(serialized, res.to_dict())



# Copyright 2019 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import IsolatedAsyncioTestCase
from unittest.mock import AsyncMock

from kubernetes_asyncio.utils import create_from_dict, create_from_yaml


class CreateFromYamlTest(IsolatedAsyncioTestCase):

    async def test_create_from_yaml(self):
        api_client = AsyncMock()
        api_client.call_api = AsyncMock()
        api_client.call_api.return_value = 'mock-value'

        created = await create_from_yaml(api_client, 'examples/nginx-deployment.yaml')

        # simple check for api call
        self.assertEqual(api_client.call_api.call_args[0][0],
                         '/apis/apps/v1/namespaces/{namespace}/deployments')

        # returned values
        self.assertEqual(created, [['mock-value']])

    async def test_create_from_dict(self):
        api_client = AsyncMock()
        api_client.call_api = AsyncMock()
        api_client.call_api.return_value = 'mock-value'

        created = await create_from_dict(api_client, {
            'apiVersion': 'apps/v1',
            'kind': 'Deployment',
            'metadata': {
                'name': 'nginx-deployment'},
            'spec': {
                'replicas': 3,
            }
        })

        # simple check for api call
        self.assertEqual(api_client.call_api.call_args[0][0],
                         '/apis/apps/v1/namespaces/{namespace}/deployments')

        # returned values
        self.assertEqual(created, ['mock-value'])



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1.32.3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from kubernetes_asyncio.client.configuration import Configuration


class V1PodDNSConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'nameservers': 'list[str]',
        'options': 'list[V1PodDNSConfigOption]',
        'searches': 'list[str]'
    }

    attribute_map = {
        'nameservers': 'nameservers',
        'options': 'options',
        'searches': 'searches'
    }

    def __init__(self, nameservers=None, options=None, searches=None, local_vars_configuration=None):  # noqa: E501
        """V1PodDNSConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default()
        self.local_vars_configuration = local_vars_configuration

        self._nameservers = None
        self._options = None
        self._searches = None
        self.discriminator = None

        if nameservers is not None:
            self.nameservers = nameservers
        if options is not None:
            self.options = options
        if searches is not None:
            self.searches = searches

    @property
    def nameservers(self):
        """Gets the nameservers of this V1PodDNSConfig.  # noqa: E501

        A list of DNS name server IP addresses. This will be appended to the base nameservers generated from DNSPolicy. Duplicated nameservers will be removed.  # noqa: E501

        :return: The nameservers of this V1PodDNSConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._nameservers

    @nameservers.setter
    def nameservers(self, nameservers):
        """Sets the nameservers of this V1PodDNSConfig.

        A list of DNS name server IP addresses. This will be appended to the base nameservers generated from DNSPolicy. Duplicated nameservers will be removed.  # noqa: E501

        :param nameservers: The nameservers of this V1PodDNSConfig.  # noqa: E501
        :type nameservers: list[str]
        """

        self._nameservers = nameservers

    @property
    def options(self):
        """Gets the options of this V1PodDNSConfig.  # noqa: E501

        A list of DNS resolver options. This will be merged with the base options generated from DNSPolicy. Duplicated entries will be removed. Resolution options given in Options will override those that appear in the base DNSPolicy.  # noqa: E501

        :return: The options of this V1PodDNSConfig.  # noqa: E501
        :rtype: list[V1PodDNSConfigOption]
        """
        return self._options

    @options.setter
    def options(self, options):
        """Sets the options of this V1PodDNSConfig.

        A list of DNS resolver options. This will be merged with the base options generated from DNSPolicy. Duplicated entries will be removed. Resolution options given in Options will override those that appear in the base DNSPolicy.  # noqa: E501

        :param options: The options of this V1PodDNSConfig.  # noqa: E501
        :type options: list[V1PodDNSConfigOption]
        """

        self._options = options

    @property
    def searches(self):
        """Gets the searches of this V1PodDNSConfig.  # noqa: E501

        A list of DNS search domains for host-name lookup. This will be appended to the base search paths generated from DNSPolicy. Duplicated search paths will be removed.  # noqa: E501

        :return: The searches of this V1PodDNSConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._searches

    @searches.setter
    def searches(self, searches):
        """Sets the searches of this V1PodDNSConfig.

        A list of DNS search domains for host-name lookup. This will be appended to the base search paths generated from DNSPolicy. Duplicated search paths will be removed.  # noqa: E501

        :param searches: The searches of this V1PodDNSConfig.  # noqa: E501
        :type searches: list[str]
        """

        self._searches = searches

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1PodDNSConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1PodDNSConfig):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1.32.3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from kubernetes_asyncio.client.configuration import Configuration


class V1beta1DeviceRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'admin_access': 'bool',
        'allocation_mode': 'str',
        'count': 'int',
        'device_class_name': 'str',
        'name': 'str',
        'selectors': 'list[V1beta1DeviceSelector]'
    }

    attribute_map = {
        'admin_access': 'adminAccess',
        'allocation_mode': 'allocationMode',
        'count': 'count',
        'device_class_name': 'deviceClassName',
        'name': 'name',
        'selectors': 'selectors'
    }

    def __init__(self, admin_access=None, allocation_mode=None, count=None, device_class_name=None, name=None, selectors=None, local_vars_configuration=None):  # noqa: E501
        """V1beta1DeviceRequest - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default()
        self.local_vars_configuration = local_vars_configuration

        self._admin_access = None
        self._allocation_mode = None
        self._count = None
        self._device_class_name = None
        self._name = None
        self._selectors = None
        self.discriminator = None

        if admin_access is not None:
            self.admin_access = admin_access
        if allocation_mode is not None:
            self.allocation_mode = allocation_mode
        if count is not None:
            self.count = count
        self.device_class_name = device_class_name
        self.name = name
        if selectors is not None:
            self.selectors = selectors

    @property
    def admin_access(self):
        """Gets the admin_access of this V1beta1DeviceRequest.  # noqa: E501

        AdminAccess indicates that this is a claim for administrative access to the device(s). Claims with AdminAccess are expected to be used for monitoring or other management services for a device.  They ignore all ordinary claims to the device with respect to access modes and any resource allocations.  This is an alpha field and requires enabling the DRAAdminAccess feature gate. Admin access is disabled if this field is unset or set to false, otherwise it is enabled.  # noqa: E501

        :return: The admin_access of this V1beta1DeviceRequest.  # noqa: E501
        :rtype: bool
        """
        return self._admin_access

    @admin_access.setter
    def admin_access(self, admin_access):
        """Sets the admin_access of this V1beta1DeviceRequest.

        AdminAccess indicates that this is a claim for administrative access to the device(s). Claims with AdminAccess are expected to be used for monitoring or other management services for a device.  They ignore all ordinary claims to the device with respect to access modes and any resource allocations.  This is an alpha field and requires enabling the DRAAdminAccess feature gate. Admin access is disabled if this field is unset or set to false, otherwise it is enabled.  # noqa: E501

        :param admin_access: The admin_access of this V1beta1DeviceRequest.  # noqa: E501
        :type admin_access: bool
        """

        self._admin_access = admin_access

    @property
    def allocation_mode(self):
        """Gets the allocation_mode of this V1beta1DeviceRequest.  # noqa: E501

        AllocationMode and its related fields define how devices are allocated to satisfy this request. Supported values are:  - ExactCount: This request is for a specific number of devices.   This is the default. The exact number is provided in the   count field.  - All: This request is for all of the matching devices in a pool.   Allocation will fail if some devices are already allocated,   unless adminAccess is requested.  If AlloctionMode is not specified, the default mode is ExactCount. If the mode is ExactCount and count is not specified, the default count is one. Any other requests must specify this field.  More modes may get added in the future. Clients must refuse to handle requests with unknown modes.  # noqa: E501

        :return: The allocation_mode of this V1beta1DeviceRequest.  # noqa: E501
        :rtype: str
        """
        return self._allocation_mode

    @allocation_mode.setter
    def allocation_mode(self, allocation_mode):
        """Sets the allocation_mode of this V1beta1DeviceRequest.

        AllocationMode and its related fields define how devices are allocated to satisfy this request. Supported values are:  - ExactCount: This request is for a specific number of devices.   This is the default. The exact number is provided in the   count field.  - All: This request is for all of the matching devices in a pool.   Allocation will fail if some devices are already allocated,   unless adminAccess is requested.  If AlloctionMode is not specified, the default mode is ExactCount. If the mode is ExactCount and count is not specified, the default count is one. Any other requests must specify this field.  More modes may get added in the future. Clients must refuse to handle requests with unknown modes.  # noqa: E501

        :param allocation_mode: The allocation_mode of this V1beta1DeviceRequest.  # noqa: E501
        :type allocation_mode: str
        """

        self._allocation_mode = allocation_mode

    @property
    def count(self):
        """Gets the count of this V1beta1DeviceRequest.  # noqa: E501

        Count is used only when the count mode is \"ExactCount\". Must be greater than zero. If AllocationMode is ExactCount and this field is not specified, the default is one.  # noqa: E501

        :return: The count of this V1beta1DeviceRequest.  # noqa: E501
        :rtype: int
        """
        return self._count

    @count.setter
    def count(self, count):
        """Sets the count of this V1beta1DeviceRequest.

        Count is used only when the count mode is \"ExactCount\". Must be greater than zero. If AllocationMode is ExactCount and this field is not specified, the default is one.  # noqa: E501

        :param count: The count of this V1beta1DeviceRequest.  # noqa: E501
        :type count: int
        """

        self._count = count

    @property
    def device_class_name(self):
        """Gets the device_class_name of this V1beta1DeviceRequest.  # noqa: E501

        DeviceClassName references a specific DeviceClass, which can define additional configuration and selectors to be inherited by this request.  A class is required. Which classes are available depends on the cluster.  Administrators may use this to restrict which devices may get requested by only installing classes with selectors for permitted devices. If users are free to request anything without restrictions, then administrators can create an empty DeviceClass for users to reference.  # noqa: E501

        :return: The device_class_name of this V1beta1DeviceRequest.  # noqa: E501
        :rtype: str
        """
        return self._device_class_name

    @device_class_name.setter
    def device_class_name(self, device_class_name):
        """Sets the device_class_name of this V1beta1DeviceRequest.

        DeviceClassName references a specific DeviceClass, which can define additional configuration and selectors to be inherited by this request.  A class is required. Which classes are available depends on the cluster.  Administrators may use this to restrict which devices may get requested by only installing classes with selectors for permitted devices. If users are free to request anything without restrictions, then administrators can create an empty DeviceClass for users to reference.  # noqa: E501

        :param device_class_name: The device_class_name of this V1beta1DeviceRequest.  # noqa: E501
        :type device_class_name: str
        """
        if self.local_vars_configuration.client_side_validation and device_class_name is None:  # noqa: E501
            raise ValueError("Invalid value for `device_class_name`, must not be `None`")  # noqa: E501

        self._device_class_name = device_class_name

    @property
    def name(self):
        """Gets the name of this V1beta1DeviceRequest.  # noqa: E501

        Name can be used to reference this request in a pod.spec.containers[].resources.claims entry and in a constraint of the claim.  Must be a DNS label.  # noqa: E501

        :return: The name of this V1beta1DeviceRequest.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this V1beta1DeviceRequest.

        Name can be used to reference this request in a pod.spec.containers[].resources.claims entry and in a constraint of the claim.  Must be a DNS label.  # noqa: E501

        :param name: The name of this V1beta1DeviceRequest.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def selectors(self):
        """Gets the selectors of this V1beta1DeviceRequest.  # noqa: E501

        Selectors define criteria which must be satisfied by a specific device in order for that device to be considered for this request. All selectors must be satisfied for a device to be considered.  # noqa: E501

        :return: The selectors of this V1beta1DeviceRequest.  # noqa: E501
        :rtype: list[V1beta1DeviceSelector]
        """
        return self._selectors

    @selectors.setter
    def selectors(self, selectors):
        """Sets the selectors of this V1beta1DeviceRequest.

        Selectors define criteria which must be satisfied by a specific device in order for that device to be considered for this request. All selectors must be satisfied for a device to be considered.  # noqa: E501

        :param selectors: The selectors of this V1beta1DeviceRequest.  # noqa: E501
        :type selectors: list[V1beta1DeviceSelector]
        """

        self._selectors = selectors

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1beta1DeviceRequest):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1beta1DeviceRequest):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1.32.3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from kubernetes_asyncio.client.configuration import Configuration


class V1alpha3DeviceAttribute(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'bool': 'bool',
        'int': 'int',
        'string': 'str',
        'version': 'str'
    }

    attribute_map = {
        'bool': 'bool',
        'int': 'int',
        'string': 'string',
        'version': 'version'
    }

    def __init__(self, bool=None, int=None, string=None, version=None, local_vars_configuration=None):  # noqa: E501
        """V1alpha3DeviceAttribute - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default()
        self.local_vars_configuration = local_vars_configuration

        self._bool = None
        self._int = None
        self._string = None
        self._version = None
        self.discriminator = None

        if bool is not None:
            self.bool = bool
        if int is not None:
            self.int = int
        if string is not None:
            self.string = string
        if version is not None:
            self.version = version

    @property
    def bool(self):
        """Gets the bool of this V1alpha3DeviceAttribute.  # noqa: E501

        BoolValue is a true/false value.  # noqa: E501

        :return: The bool of this V1alpha3DeviceAttribute.  # noqa: E501
        :rtype: bool
        """
        return self._bool

    @bool.setter
    def bool(self, bool):
        """Sets the bool of this V1alpha3DeviceAttribute.

        BoolValue is a true/false value.  # noqa: E501

        :param bool: The bool of this V1alpha3DeviceAttribute.  # noqa: E501
        :type bool: bool
        """

        self._bool = bool

    @property
    def int(self):
        """Gets the int of this V1alpha3DeviceAttribute.  # noqa: E501

        IntValue is a number.  # noqa: E501

        :return: The int of this V1alpha3DeviceAttribute.  # noqa: E501
        :rtype: int
        """
        return self._int

    @int.setter
    def int(self, int):
        """Sets the int of this V1alpha3DeviceAttribute.

        IntValue is a number.  # noqa: E501

        :param int: The int of this V1alpha3DeviceAttribute.  # noqa: E501
        :type int: int
        """

        self._int = int

    @property
    def string(self):
        """Gets the string of this V1alpha3DeviceAttribute.  # noqa: E501

        StringValue is a string. Must not be longer than 64 characters.  # noqa: E501

        :return: The string of this V1alpha3DeviceAttribute.  # noqa: E501
        :rtype: str
        """
        return self._string

    @string.setter
    def string(self, string):
        """Sets the string of this V1alpha3DeviceAttribute.

        StringValue is a string. Must not be longer than 64 characters.  # noqa: E501

        :param string: The string of this V1alpha3DeviceAttribute.  # noqa: E501
        :type string: str
        """

        self._string = string

    @property
    def version(self):
        """Gets the version of this V1alpha3DeviceAttribute.  # noqa: E501

        VersionValue is a semantic version according to semver.org spec 2.0.0. Must not be longer than 64 characters.  # noqa: E501

        :return: The version of this V1alpha3DeviceAttribute.  # noqa: E501
        :rtype: str
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this V1alpha3DeviceAttribute.

        VersionValue is a semantic version according to semver.org spec 2.0.0. Must not be longer than 64 characters.  # noqa: E501

        :param version: The version of this V1alpha3DeviceAttribute.  # noqa: E501
        :type version: str
        """

        self._version = version

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1alpha3DeviceAttribute):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1alpha3DeviceAttribute):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1.32.3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from kubernetes_asyncio.client.configuration import Configuration


class V1alpha1StorageVersionMigrationSpec(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'continue_token': 'str',
        'resource': 'V1alpha1GroupVersionResource'
    }

    attribute_map = {
        'continue_token': 'continueToken',
        'resource': 'resource'
    }

    def __init__(self, continue_token=None, resource=None, local_vars_configuration=None):  # noqa: E501
        """V1alpha1StorageVersionMigrationSpec - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default()
        self.local_vars_configuration = local_vars_configuration

        self._continue_token = None
        self._resource = None
        self.discriminator = None

        if continue_token is not None:
            self.continue_token = continue_token
        self.resource = resource

    @property
    def continue_token(self):
        """Gets the continue_token of this V1alpha1StorageVersionMigrationSpec.  # noqa: E501

        The token used in the list options to get the next chunk of objects to migrate. When the .status.conditions indicates the migration is \"Running\", users can use this token to check the progress of the migration.  # noqa: E501

        :return: The continue_token of this V1alpha1StorageVersionMigrationSpec.  # noqa: E501
        :rtype: str
        """
        return self._continue_token

    @continue_token.setter
    def continue_token(self, continue_token):
        """Sets the continue_token of this V1alpha1StorageVersionMigrationSpec.

        The token used in the list options to get the next chunk of objects to migrate. When the .status.conditions indicates the migration is \"Running\", users can use this token to check the progress of the migration.  # noqa: E501

        :param continue_token: The continue_token of this V1alpha1StorageVersionMigrationSpec.  # noqa: E501
        :type continue_token: str
        """

        self._continue_token = continue_token

    @property
    def resource(self):
        """Gets the resource of this V1alpha1StorageVersionMigrationSpec.  # noqa: E501


        :return: The resource of this V1alpha1StorageVersionMigrationSpec.  # noqa: E501
        :rtype: V1alpha1GroupVersionResource
        """
        return self._resource

    @resource.setter
    def resource(self, resource):
        """Sets the resource of this V1alpha1StorageVersionMigrationSpec.


        :param resource: The resource of this V1alpha1StorageVersionMigrationSpec.  # noqa: E501
        :type resource: V1alpha1GroupVersionResource
        """
        if self.local_vars_configuration.client_side_validation and resource is None:  # noqa: E501
            raise ValueError("Invalid value for `resource`, must not be `None`")  # noqa: E501

        self._resource = resource

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1alpha1StorageVersionMigrationSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1alpha1StorageVersionMigrationSpec):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1.32.3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from kubernetes_asyncio.client.configuration import Configuration


class V1LinuxContainerUser(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'gid': 'int',
        'supplemental_groups': 'list[int]',
        'uid': 'int'
    }

    attribute_map = {
        'gid': 'gid',
        'supplemental_groups': 'supplementalGroups',
        'uid': 'uid'
    }

    def __init__(self, gid=None, supplemental_groups=None, uid=None, local_vars_configuration=None):  # noqa: E501
        """V1LinuxContainerUser - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default()
        self.local_vars_configuration = local_vars_configuration

        self._gid = None
        self._supplemental_groups = None
        self._uid = None
        self.discriminator = None

        self.gid = gid
        if supplemental_groups is not None:
            self.supplemental_groups = supplemental_groups
        self.uid = uid

    @property
    def gid(self):
        """Gets the gid of this V1LinuxContainerUser.  # noqa: E501

        GID is the primary gid initially attached to the first process in the container  # noqa: E501

        :return: The gid of this V1LinuxContainerUser.  # noqa: E501
        :rtype: int
        """
        return self._gid

    @gid.setter
    def gid(self, gid):
        """Sets the gid of this V1LinuxContainerUser.

        GID is the primary gid initially attached to the first process in the container  # noqa: E501

        :param gid: The gid of this V1LinuxContainerUser.  # noqa: E501
        :type gid: int
        """
        if self.local_vars_configuration.client_side_validation and gid is None:  # noqa: E501
            raise ValueError("Invalid value for `gid`, must not be `None`")  # noqa: E501

        self._gid = gid

    @property
    def supplemental_groups(self):
        """Gets the supplemental_groups of this V1LinuxContainerUser.  # noqa: E501

        SupplementalGroups are the supplemental groups initially attached to the first process in the container  # noqa: E501

        :return: The supplemental_groups of this V1LinuxContainerUser.  # noqa: E501
        :rtype: list[int]
        """
        return self._supplemental_groups

    @supplemental_groups.setter
    def supplemental_groups(self, supplemental_groups):
        """Sets the supplemental_groups of this V1LinuxContainerUser.

        SupplementalGroups are the supplemental groups initially attached to the first process in the container  # noqa: E501

        :param supplemental_groups: The supplemental_groups of this V1LinuxContainerUser.  # noqa: E501
        :type supplemental_groups: list[int]
        """

        self._supplemental_groups = supplemental_groups

    @property
    def uid(self):
        """Gets the uid of this V1LinuxContainerUser.  # noqa: E501

        UID is the primary uid initially attached to the first process in the container  # noqa: E501

        :return: The uid of this V1LinuxContainerUser.  # noqa: E501
        :rtype: int
        """
        return self._uid

    @uid.setter
    def uid(self, uid):
        """Sets the uid of this V1LinuxContainerUser.

        UID is the primary uid initially attached to the first process in the container  # noqa: E501

        :param uid: The uid of this V1LinuxContainerUser.  # noqa: E501
        :type uid: int
        """
        if self.local_vars_configuration.client_side_validation and uid is None:  # noqa: E501
            raise ValueError("Invalid value for `uid`, must not be `None`")  # noqa: E501

        self._uid = uid

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1LinuxContainerUser):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1LinuxContainerUser):
            return True

        return self.to_dict() != other.to_dict()



import asyncio
import ssl
import unittest
from unittest.mock import AsyncMock
import aiohttp
from kubernetes_asyncio.client.rest import RESTClientObject
from kubernetes_asyncio.client.configuration import Configuration
class TestRESTClientObject(unittest.IsolatedAsyncioTestCase):

    @classmethod
    def setUpClass(cls):
        cls.config = Configuration()


    async def test_rest_request_timeout(self):
        rest_api = RESTClientObject(configuration=self.config)
        for request_timeout, expected_timeout_arg in [
                (None, aiohttp.ClientTimeout()),
                (5.0, aiohttp.ClientTimeout(total=5.0)),
                (3, aiohttp.ClientTimeout(total=3)),
                ((5, 7), aiohttp.ClientTimeout(connect=5, sock_connect=5, sock_read=7)),
                (aiohttp.ClientTimeout(total=None), aiohttp.ClientTimeout(total=None)),
        ]:
            with self.subTest(request_timeout=request_timeout, expected_timeout_arg=expected_timeout_arg):
                mock_request = AsyncMock()
                rest_api.pool_manager.request = mock_request
                await rest_api.request(method="GET", url="http://test-api", _preload_content=False, _request_timeout=request_timeout)
                mock_request.assert_called_once_with(
                    method="GET",
                    url="http://test-api",
                    timeout=expected_timeout_arg,
                    headers={"Content-Type": "application/json"}
                )

    async def test_disable_ssl_verification(self):
        configuration = Configuration()
        configuration.disable_strict_ssl_verification = True
        rest_api = RESTClientObject(configuration=configuration)
        ssl_context = rest_api.pool_manager._connector._ssl
        self.assertEqual(ssl_context.verify_flags & ssl.VERIFY_X509_STRICT, 0)



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1.32.3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from kubernetes_asyncio.client.configuration import Configuration


class V1KeyToPath(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'key': 'str',
        'mode': 'int',
        'path': 'str'
    }

    attribute_map = {
        'key': 'key',
        'mode': 'mode',
        'path': 'path'
    }

    def __init__(self, key=None, mode=None, path=None, local_vars_configuration=None):  # noqa: E501
        """V1KeyToPath - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default()
        self.local_vars_configuration = local_vars_configuration

        self._key = None
        self._mode = None
        self._path = None
        self.discriminator = None

        self.key = key
        if mode is not None:
            self.mode = mode
        self.path = path

    @property
    def key(self):
        """Gets the key of this V1KeyToPath.  # noqa: E501

        key is the key to project.  # noqa: E501

        :return: The key of this V1KeyToPath.  # noqa: E501
        :rtype: str
        """
        return self._key

    @key.setter
    def key(self, key):
        """Sets the key of this V1KeyToPath.

        key is the key to project.  # noqa: E501

        :param key: The key of this V1KeyToPath.  # noqa: E501
        :type key: str
        """
        if self.local_vars_configuration.client_side_validation and key is None:  # noqa: E501
            raise ValueError("Invalid value for `key`, must not be `None`")  # noqa: E501

        self._key = key

    @property
    def mode(self):
        """Gets the mode of this V1KeyToPath.  # noqa: E501

        mode is Optional: mode bits used to set permissions on this file. Must be an octal value between 0000 and 0777 or a decimal value between 0 and 511. YAML accepts both octal and decimal values, JSON requires decimal values for mode bits. If not specified, the volume defaultMode will be used. This might be in conflict with other options that affect the file mode, like fsGroup, and the result can be other mode bits set.  # noqa: E501

        :return: The mode of this V1KeyToPath.  # noqa: E501
        :rtype: int
        """
        return self._mode

    @mode.setter
    def mode(self, mode):
        """Sets the mode of this V1KeyToPath.

        mode is Optional: mode bits used to set permissions on this file. Must be an octal value between 0000 and 0777 or a decimal value between 0 and 511. YAML accepts both octal and decimal values, JSON requires decimal values for mode bits. If not specified, the volume defaultMode will be used. This might be in conflict with other options that affect the file mode, like fsGroup, and the result can be other mode bits set.  # noqa: E501

        :param mode: The mode of this V1KeyToPath.  # noqa: E501
        :type mode: int
        """

        self._mode = mode

    @property
    def path(self):
        """Gets the path of this V1KeyToPath.  # noqa: E501

        path is the relative path of the file to map the key to. May not be an absolute path. May not contain the path element '..'. May not start with the string '..'.  # noqa: E501

        :return: The path of this V1KeyToPath.  # noqa: E501
        :rtype: str
        """
        return self._path

    @path.setter
    def path(self, path):
        """Sets the path of this V1KeyToPath.

        path is the relative path of the file to map the key to. May not be an absolute path. May not contain the path element '..'. May not start with the string '..'.  # noqa: E501

        :param path: The path of this V1KeyToPath.  # noqa: E501
        :type path: str
        """
        if self.local_vars_configuration.client_side_validation and path is None:  # noqa: E501
            raise ValueError("Invalid value for `path`, must not be `None`")  # noqa: E501

        self._path = path

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1KeyToPath):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1KeyToPath):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1.32.3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from kubernetes_asyncio.client.configuration import Configuration


class V1ContainerStatus(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'allocated_resources': 'dict(str, str)',
        'allocated_resources_status': 'list[V1ResourceStatus]',
        'container_id': 'str',
        'image': 'str',
        'image_id': 'str',
        'last_state': 'V1ContainerState',
        'name': 'str',
        'ready': 'bool',
        'resources': 'V1ResourceRequirements',
        'restart_count': 'int',
        'started': 'bool',
        'state': 'V1ContainerState',
        'user': 'V1ContainerUser',
        'volume_mounts': 'list[V1VolumeMountStatus]'
    }

    attribute_map = {
        'allocated_resources': 'allocatedResources',
        'allocated_resources_status': 'allocatedResourcesStatus',
        'container_id': 'containerID',
        'image': 'image',
        'image_id': 'imageID',
        'last_state': 'lastState',
        'name': 'name',
        'ready': 'ready',
        'resources': 'resources',
        'restart_count': 'restartCount',
        'started': 'started',
        'state': 'state',
        'user': 'user',
        'volume_mounts': 'volumeMounts'
    }

    def __init__(self, allocated_resources=None, allocated_resources_status=None, container_id=None, image=None, image_id=None, last_state=None, name=None, ready=None, resources=None, restart_count=None, started=None, state=None, user=None, volume_mounts=None, local_vars_configuration=None):  # noqa: E501
        """V1ContainerStatus - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default()
        self.local_vars_configuration = local_vars_configuration

        self._allocated_resources = None
        self._allocated_resources_status = None
        self._container_id = None
        self._image = None
        self._image_id = None
        self._last_state = None
        self._name = None
        self._ready = None
        self._resources = None
        self._restart_count = None
        self._started = None
        self._state = None
        self._user = None
        self._volume_mounts = None
        self.discriminator = None

        if allocated_resources is not None:
            self.allocated_resources = allocated_resources
        if allocated_resources_status is not None:
            self.allocated_resources_status = allocated_resources_status
        if container_id is not None:
            self.container_id = container_id
        self.image = image
        self.image_id = image_id
        if last_state is not None:
            self.last_state = last_state
        self.name = name
        self.ready = ready
        if resources is not None:
            self.resources = resources
        self.restart_count = restart_count
        if started is not None:
            self.started = started
        if state is not None:
            self.state = state
        if user is not None:
            self.user = user
        if volume_mounts is not None:
            self.volume_mounts = volume_mounts

    @property
    def allocated_resources(self):
        """Gets the allocated_resources of this V1ContainerStatus.  # noqa: E501

        AllocatedResources represents the compute resources allocated for this container by the node. Kubelet sets this value to Container.Resources.Requests upon successful pod admission and after successfully admitting desired pod resize.  # noqa: E501

        :return: The allocated_resources of this V1ContainerStatus.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._allocated_resources

    @allocated_resources.setter
    def allocated_resources(self, allocated_resources):
        """Sets the allocated_resources of this V1ContainerStatus.

        AllocatedResources represents the compute resources allocated for this container by the node. Kubelet sets this value to Container.Resources.Requests upon successful pod admission and after successfully admitting desired pod resize.  # noqa: E501

        :param allocated_resources: The allocated_resources of this V1ContainerStatus.  # noqa: E501
        :type allocated_resources: dict(str, str)
        """

        self._allocated_resources = allocated_resources

    @property
    def allocated_resources_status(self):
        """Gets the allocated_resources_status of this V1ContainerStatus.  # noqa: E501

        AllocatedResourcesStatus represents the status of various resources allocated for this Pod.  # noqa: E501

        :return: The allocated_resources_status of this V1ContainerStatus.  # noqa: E501
        :rtype: list[V1ResourceStatus]
        """
        return self._allocated_resources_status

    @allocated_resources_status.setter
    def allocated_resources_status(self, allocated_resources_status):
        """Sets the allocated_resources_status of this V1ContainerStatus.

        AllocatedResourcesStatus represents the status of various resources allocated for this Pod.  # noqa: E501

        :param allocated_resources_status: The allocated_resources_status of this V1ContainerStatus.  # noqa: E501
        :type allocated_resources_status: list[V1ResourceStatus]
        """

        self._allocated_resources_status = allocated_resources_status

    @property
    def container_id(self):
        """Gets the container_id of this V1ContainerStatus.  # noqa: E501

        ContainerID is the ID of the container in the format '<type>://<container_id>'. Where type is a container runtime identifier, returned from Version call of CRI API (for example \"containerd\").  # noqa: E501

        :return: The container_id of this V1ContainerStatus.  # noqa: E501
        :rtype: str
        """
        return self._container_id

    @container_id.setter
    def container_id(self, container_id):
        """Sets the container_id of this V1ContainerStatus.

        ContainerID is the ID of the container in the format '<type>://<container_id>'. Where type is a container runtime identifier, returned from Version call of CRI API (for example \"containerd\").  # noqa: E501

        :param container_id: The container_id of this V1ContainerStatus.  # noqa: E501
        :type container_id: str
        """

        self._container_id = container_id

    @property
    def image(self):
        """Gets the image of this V1ContainerStatus.  # noqa: E501

        Image is the name of container image that the container is running. The container image may not match the image used in the PodSpec, as it may have been resolved by the runtime. More info: https://kubernetes.io/docs/concepts/containers/images.  # noqa: E501

        :return: The image of this V1ContainerStatus.  # noqa: E501
        :rtype: str
        """
        return self._image

    @image.setter
    def image(self, image):
        """Sets the image of this V1ContainerStatus.

        Image is the name of container image that the container is running. The container image may not match the image used in the PodSpec, as it may have been resolved by the runtime. More info: https://kubernetes.io/docs/concepts/containers/images.  # noqa: E501

        :param image: The image of this V1ContainerStatus.  # noqa: E501
        :type image: str
        """
        if self.local_vars_configuration.client_side_validation and image is None:  # noqa: E501
            raise ValueError("Invalid value for `image`, must not be `None`")  # noqa: E501

        self._image = image

    @property
    def image_id(self):
        """Gets the image_id of this V1ContainerStatus.  # noqa: E501

        ImageID is the image ID of the container's image. The image ID may not match the image ID of the image used in the PodSpec, as it may have been resolved by the runtime.  # noqa: E501

        :return: The image_id of this V1ContainerStatus.  # noqa: E501
        :rtype: str
        """
        return self._image_id

    @image_id.setter
    def image_id(self, image_id):
        """Sets the image_id of this V1ContainerStatus.

        ImageID is the image ID of the container's image. The image ID may not match the image ID of the image used in the PodSpec, as it may have been resolved by the runtime.  # noqa: E501

        :param image_id: The image_id of this V1ContainerStatus.  # noqa: E501
        :type image_id: str
        """
        if self.local_vars_configuration.client_side_validation and image_id is None:  # noqa: E501
            raise ValueError("Invalid value for `image_id`, must not be `None`")  # noqa: E501

        self._image_id = image_id

    @property
    def last_state(self):
        """Gets the last_state of this V1ContainerStatus.  # noqa: E501


        :return: The last_state of this V1ContainerStatus.  # noqa: E501
        :rtype: V1ContainerState
        """
        return self._last_state

    @last_state.setter
    def last_state(self, last_state):
        """Sets the last_state of this V1ContainerStatus.


        :param last_state: The last_state of this V1ContainerStatus.  # noqa: E501
        :type last_state: V1ContainerState
        """

        self._last_state = last_state

    @property
    def name(self):
        """Gets the name of this V1ContainerStatus.  # noqa: E501

        Name is a DNS_LABEL representing the unique name of the container. Each container in a pod must have a unique name across all container types. Cannot be updated.  # noqa: E501

        :return: The name of this V1ContainerStatus.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this V1ContainerStatus.

        Name is a DNS_LABEL representing the unique name of the container. Each container in a pod must have a unique name across all container types. Cannot be updated.  # noqa: E501

        :param name: The name of this V1ContainerStatus.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def ready(self):
        """Gets the ready of this V1ContainerStatus.  # noqa: E501

        Ready specifies whether the container is currently passing its readiness check. The value will change as readiness probes keep executing. If no readiness probes are specified, this field defaults to true once the container is fully started (see Started field).  The value is typically used to determine whether a container is ready to accept traffic.  # noqa: E501

        :return: The ready of this V1ContainerStatus.  # noqa: E501
        :rtype: bool
        """
        return self._ready

    @ready.setter
    def ready(self, ready):
        """Sets the ready of this V1ContainerStatus.

        Ready specifies whether the container is currently passing its readiness check. The value will change as readiness probes keep executing. If no readiness probes are specified, this field defaults to true once the container is fully started (see Started field).  The value is typically used to determine whether a container is ready to accept traffic.  # noqa: E501

        :param ready: The ready of this V1ContainerStatus.  # noqa: E501
        :type ready: bool
        """
        if self.local_vars_configuration.client_side_validation and ready is None:  # noqa: E501
            raise ValueError("Invalid value for `ready`, must not be `None`")  # noqa: E501

        self._ready = ready

    @property
    def resources(self):
        """Gets the resources of this V1ContainerStatus.  # noqa: E501


        :return: The resources of this V1ContainerStatus.  # noqa: E501
        :rtype: V1ResourceRequirements
        """
        return self._resources

    @resources.setter
    def resources(self, resources):
        """Sets the resources of this V1ContainerStatus.


        :param resources: The resources of this V1ContainerStatus.  # noqa: E501
        :type resources: V1ResourceRequirements
        """

        self._resources = resources

    @property
    def restart_count(self):
        """Gets the restart_count of this V1ContainerStatus.  # noqa: E501

        RestartCount holds the number of times the container has been restarted. Kubelet makes an effort to always increment the value, but there are cases when the state may be lost due to node restarts and then the value may be reset to 0. The value is never negative.  # noqa: E501

        :return: The restart_count of this V1ContainerStatus.  # noqa: E501
        :rtype: int
        """
        return self._restart_count

    @restart_count.setter
    def restart_count(self, restart_count):
        """Sets the restart_count of this V1ContainerStatus.

        RestartCount holds the number of times the container has been restarted. Kubelet makes an effort to always increment the value, but there are cases when the state may be lost due to node restarts and then the value may be reset to 0. The value is never negative.  # noqa: E501

        :param restart_count: The restart_count of this V1ContainerStatus.  # noqa: E501
        :type restart_count: int
        """
        if self.local_vars_configuration.client_side_validation and restart_count is None:  # noqa: E501
            raise ValueError("Invalid value for `restart_count`, must not be `None`")  # noqa: E501

        self._restart_count = restart_count

    @property
    def started(self):
        """Gets the started of this V1ContainerStatus.  # noqa: E501

        Started indicates whether the container has finished its postStart lifecycle hook and passed its startup probe. Initialized as false, becomes true after startupProbe is considered successful. Resets to false when the container is restarted, or if kubelet loses state temporarily. In both cases, startup probes will run again. Is always true when no startupProbe is defined and container is running and has passed the postStart lifecycle hook. The null value must be treated the same as false.  # noqa: E501

        :return: The started of this V1ContainerStatus.  # noqa: E501
        :rtype: bool
        """
        return self._started

    @started.setter
    def started(self, started):
        """Sets the started of this V1ContainerStatus.

        Started indicates whether the container has finished its postStart lifecycle hook and passed its startup probe. Initialized as false, becomes true after startupProbe is considered successful. Resets to false when the container is restarted, or if kubelet loses state temporarily. In both cases, startup probes will run again. Is always true when no startupProbe is defined and container is running and has passed the postStart lifecycle hook. The null value must be treated the same as false.  # noqa: E501

        :param started: The started of this V1ContainerStatus.  # noqa: E501
        :type started: bool
        """

        self._started = started

    @property
    def state(self):
        """Gets the state of this V1ContainerStatus.  # noqa: E501


        :return: The state of this V1ContainerStatus.  # noqa: E501
        :rtype: V1ContainerState
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this V1ContainerStatus.


        :param state: The state of this V1ContainerStatus.  # noqa: E501
        :type state: V1ContainerState
        """

        self._state = state

    @property
    def user(self):
        """Gets the user of this V1ContainerStatus.  # noqa: E501


        :return: The user of this V1ContainerStatus.  # noqa: E501
        :rtype: V1ContainerUser
        """
        return self._user

    @user.setter
    def user(self, user):
        """Sets the user of this V1ContainerStatus.


        :param user: The user of this V1ContainerStatus.  # noqa: E501
        :type user: V1ContainerUser
        """

        self._user = user

    @property
    def volume_mounts(self):
        """Gets the volume_mounts of this V1ContainerStatus.  # noqa: E501

        Status of volume mounts.  # noqa: E501

        :return: The volume_mounts of this V1ContainerStatus.  # noqa: E501
        :rtype: list[V1VolumeMountStatus]
        """
        return self._volume_mounts

    @volume_mounts.setter
    def volume_mounts(self, volume_mounts):
        """Sets the volume_mounts of this V1ContainerStatus.

        Status of volume mounts.  # noqa: E501

        :param volume_mounts: The volume_mounts of this V1ContainerStatus.  # noqa: E501
        :type volume_mounts: list[V1VolumeMountStatus]
        """

        self._volume_mounts = volume_mounts

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1ContainerStatus):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1ContainerStatus):
            return True

        return self.to_dict() != other.to_dict()
